package com.labwork1.app.student.controller;

import com.labwork1.app.student.model.Customer;
import com.labwork1.app.student.model.Order;
import com.labwork1.app.student.model.SessionExtension;
import com.labwork1.app.student.model.UserRole;
import com.labwork1.app.student.service.CustomerService;
import com.labwork1.app.student.service.OrderService;
import com.labwork1.app.student.service.SessionService;
import jakarta.validation.Valid;
import org.slf4j.event.KeyValuePair;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import java.security.Principal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

@Controller
@RequestMapping("/order")
public class OrderMvcController {
    private final OrderService orderService;
    private final SessionService sessionService;
    private final CustomerService customerService;

    public OrderMvcController(OrderService orderService, SessionService sessionService, CustomerService customerService) {
        this.orderService = orderService;
        this.sessionService = sessionService;
        this.customerService = customerService;
    }

    @GetMapping
    public String getOrders(Model model,
                            Principal principal) {
        Customer customer = customerService.findByLogin(principal.getName());
        if (customer.getRole() == UserRole.USER)
            model.addAttribute("orders",
                    orderService.findAllOrders().stream()
                            .map(OrderDto::new)
                            .filter(orderDto -> Objects.equals(orderDto.getCustomerId(),
                                    customer.getId()))
                            .toList());
        else
            model.addAttribute("orders",
                    orderService.findAllOrders().stream()
                            .map(OrderDto::new)
                            .toList());
        return "order";
    }

    @GetMapping(value = {"/edit", "/edit/{id}"})
    public String editOrder(@PathVariable(required = false) Long id,
                            Model model,
                            Principal principal) {
        if (id == null || id <= 0) {
            model.addAttribute("orderDto", new OrderDto());

            return "order-edit";
        } else {
            Order order = orderService.findOrder(id);
            Customer customer = customerService.findByLogin(principal.getName());
            if (!Objects.equals(order.getCustomer().getId(), customer.getId()) && customer.getRole() == UserRole.USER)
                return "redirect:/order";
            List<OrderSessionDto> temp = order.getSessions()
                    .stream().map(x -> new OrderSessionDto(new SessionDto(x.getSession()),
                            x.getOrder().getId(), x.getCount())).toList();
            List<SessionDto> sessions = sessionService.findAllSessions().stream()
                    .map(SessionDto::new)
                    .toList();
            HashMap<SessionDto, Integer> orderSessions = new HashMap<>();
            for (var os : temp) {
                orderSessions.put(os.getSession(), os.getCount());
            }
            model.addAttribute("orderSessions", orderSessions);
            model.addAttribute("sessions", sessions);
            model.addAttribute("orderSessionDto", new OrderSessionDto());

            return "ordersession";
        }
    }

    @PostMapping(value = "/")
    public String saveOrder(@ModelAttribute @Valid OrderDto orderDto,
                            BindingResult bindingResult,
                            Model model,
                            Principal principal) {
        Long customerId = customerService.findByLogin(principal.getName()).getId();
        if (bindingResult.hasErrors()) {
            model.addAttribute("errors", bindingResult.getAllErrors());
            return "order-edit";
        }
        orderService.addOrder(customerId);
        return "redirect:/order";
    }

    @PostMapping(value = {"/{id}"})
    public String editOrder(@PathVariable Long id,
                            @RequestParam("session") Long session,
                            @RequestParam(value = "count", required = false) Integer count,
                            Model model,
                            Principal principal) {
        Order order = orderService.findOrder(id);
        Customer customer = customerService.findByLogin(principal.getName());
        if (!Objects.equals(order.getCustomer().getId(), customer.getId()) && customer.getRole() == UserRole.USER)
            return "/order";
        if (count == null)
            orderService.deleteSessionInOrder(id, session, Integer.MAX_VALUE);
        else if (count > 0)
            orderService.addSession(id, session, count);
        return "redirect:/order/edit/" + id;
    }

    @PostMapping("/delete/{id}")
    public String deleteOrder(@PathVariable Long id,
                              Principal principal) {
        Order order = orderService.findOrder(id);
        Customer customer = customerService.findByLogin(principal.getName());
        if (!Objects.equals(order.getCustomer().getId(), customer.getId()) && customer.getRole() == UserRole.USER)
            return "redirect:/order";
        orderService.deleteOrder(id);
        return "redirect:/order";
    }

}
