package com.labwork1.app.student.service;

import com.labwork1.app.student.model.Customer;
import com.labwork1.app.student.model.UserRole;
import com.labwork1.app.student.model.UserSignupDto;
import com.labwork1.app.student.repository.CustomerRepository;
import com.labwork1.app.util.validation.ValidatorUtil;
import jakarta.validation.ValidationException;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

@Service
public class CustomerService implements UserDetailsService {
    private final CustomerRepository customerRepository;
    private final PasswordEncoder passwordEncoder;
    private final ValidatorUtil validatorUtil;

    public CustomerService(CustomerRepository customerRepository, PasswordEncoder passwordEncoder, ValidatorUtil validatorUtil) {
        this.customerRepository = customerRepository;
        this.passwordEncoder = passwordEncoder;
        this.validatorUtil = validatorUtil;
    }

    public Page<Customer> findAllPages(int page, int size) {
        return customerRepository.findAll(PageRequest.of(page - 1, size, Sort.by("id").ascending()));
    }

    public Customer findByLogin(String login) {
        return customerRepository.findOneByLoginIgnoreCase(login);
    }

    @Transactional
    public Customer addCustomer(String login, String password, String passwordConfirm) {
        return createUser(login, password, passwordConfirm, UserRole.USER);
    }

    @Transactional
    public Customer addCustomer(UserSignupDto userSignupDto) {
        if (findByLogin(userSignupDto.getLogin()) != null) {
            throw new ValidationException(String.format("User '%s' already exists", userSignupDto.getLogin()));
        }
        if (!Objects.equals(userSignupDto.getPassword(), userSignupDto.getPasswordConfirm())) {
            throw new ValidationException("Passwords not equals");
        }
        final Customer user = new Customer(userSignupDto);
        validatorUtil.validate(user);
        return customerRepository.save(user);
    }

    public Customer createUser(String login, String password, String passwordConfirm, UserRole role) {
        if (findByLogin(login) != null) {
            throw new ValidationException(String.format("User '%s' already exists", login));
        }
        final Customer user = new Customer(login, passwordEncoder.encode(password), role);
        validatorUtil.validate(user);
        if (!Objects.equals(password, passwordConfirm)) {
            throw new ValidationException("Passwords not equals");
        }
        return customerRepository.save(user);
    }

    @Transactional(readOnly = true)
    public Customer findCustomer(Long id) {
        final Optional<Customer> customer = customerRepository.findById(id);
        return customer.orElseThrow(() -> new CustomerNotFoundException(id));
    }

    @Transactional(readOnly = true)
    public List<Customer> findAllCustomers() {
        return customerRepository.findAll();
    }

    @Transactional
    public Customer updateCustomer(Long id, String login, String password) {
        final Customer currentCustomer = findCustomer(id);
        currentCustomer.setLogin(login);
        currentCustomer.setPassword(password);
        validatorUtil.validate(currentCustomer);
        return customerRepository.save(currentCustomer);
    }

    @Transactional
    public Customer deleteCustomer(Long id) {
        final Customer customer = findCustomer(id);
        customerRepository.deleteById(id);
        return customer;
    }

    @Transactional
    public void deleteAllCustomers() {
        customerRepository.deleteAll();
    }

    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        final Customer userEntity = findByLogin(username);
        if (userEntity == null) {
            throw new UsernameNotFoundException(username);
        }
        return new org.springframework.security.core.userdetails.User(
                userEntity.getLogin(), userEntity.getPassword(), Collections.singleton(userEntity.getRole()));
    }
}
