package com.labwork1.app.student.service;

import com.labwork1.app.student.model.*;
import com.labwork1.app.student.repository.OrderRepository;
import com.labwork1.app.util.validation.ValidatorUtil;
import jakarta.persistence.EntityNotFoundException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.sql.Date;
import java.util.List;
import java.util.Optional;

@Service
public class OrderService {
    private final OrderRepository orderRepository;
    private final CustomerService customerService;
    private final SessionService sessionService;
    private final ValidatorUtil validatorUtil;

    public OrderService(OrderRepository orderRepository, CustomerService customerService, SessionService sessionService, ValidatorUtil validatorUtil) {
        this.orderRepository = orderRepository;
        this.customerService = customerService;
        this.sessionService = sessionService;
        this.validatorUtil = validatorUtil;
    }

    @Transactional
    public Order addOrder(Long customerId) {
        final Order order = new Order(new Date(System.currentTimeMillis()));
        final Customer customer = customerService.findCustomer(customerId);
        order.setCustomer(customer);
        validatorUtil.validate(order);
        return orderRepository.save(order);
    }

    @Transactional
    public Order addSession(Long id, Long sessionId, Integer count) {
        final Session currentSession = sessionService.findSession(sessionId);
        final Order currentOrder = findOrder(id);
        final OrderSession currentOrderSession = orderRepository.getOrderSession(id, sessionId);

        final Integer currentSessionCapacity = currentSession.getMaxCount() - sessionService.getCapacity(sessionId);
        if (currentSessionCapacity < count ||
                (currentOrderSession != null && currentOrderSession.getCount() + count > currentSession.getMaxCount())) {
            throw new IllegalArgumentException(String.format("No more tickets in session. Capacity: %1$s. Count: %2$s",
                    currentSessionCapacity, count));
        }

        if (currentOrderSession == null) {
            currentOrder.addSession(new OrderSession(currentOrder, currentSession, count));
        }
        else if (currentOrderSession.getCount() + count <= currentSession.getMaxCount()) {
            currentOrder.removeSession(currentOrderSession);
            currentOrder.addSession(new OrderSession(currentOrder, currentSession,
                    currentOrderSession.getCount() + count));
        }

        return orderRepository.save(currentOrder);
    }

    @Transactional(readOnly = true)
    public Order findOrder(Long id) {
        final Optional<Order> order = orderRepository.findById(id);
        return order.orElseThrow(() -> new OrderNotFoundException(id));
    }

    @Transactional(readOnly = true)
    public List<Order> findAllOrders() {
        return orderRepository.findAll();
    }

    @Transactional
    public Order deleteOrder(Long id) {
        final Order currentOrder = findOrder(id);
        orderRepository.delete(currentOrder);
        return currentOrder;
    }

    @Transactional
    public Order deleteSessionInOrder(Long id, Long session, Integer count) {
        final Order currentOrder = findOrder(id);
        final Session currentSession = sessionService.findSession(session);
        final OrderSession currentOrderSession = orderRepository.getOrderSession(id, session);
        if (currentOrderSession == null)
            throw new EntityNotFoundException();

        if (count >= currentOrderSession.getCount()) {
            currentOrder.removeSession(currentOrderSession);
        }
        else {
            currentOrder.removeSession(currentOrderSession);
            currentOrder.addSession(new OrderSession(currentOrder, currentSession,
                    currentOrderSession.getCount() - count));
        }
        return orderRepository.save(currentOrder);
    }

    @Transactional
    public void deleteAllOrders() {
        orderRepository.deleteAll();
    }
}
