package com.labwork1.app.student.service;

import com.labwork1.app.student.model.Cinema;
import com.labwork1.app.student.model.Session;
import com.labwork1.app.student.model.SessionExtension;
import com.labwork1.app.student.repository.SessionRepository;
import com.labwork1.app.util.validation.ValidatorUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.sql.Timestamp;
import java.time.LocalDateTime;
import java.util.List;

@Service
public class SessionService {
    private final SessionRepository sessionRepository;
    private final CinemaService cinemaService;
    private final ValidatorUtil validatorUtil;
    public SessionService(SessionRepository sessionRepository,
                          CinemaService cinemaService, ValidatorUtil validatorUtil) {
        this.sessionRepository = sessionRepository;
        this.cinemaService = cinemaService;
        this.validatorUtil = validatorUtil;
    }

    @Transactional
    public Session addSession(Double price, LocalDateTime date, Long cinemaId, Integer capacity) {
        final Session session = new Session(price, date, capacity);
        final Cinema cinema = cinemaService.findCinema(cinemaId);
        session.setCinema(cinema);
        validatorUtil.validate(session);
        return sessionRepository.save(session);
    }

    @Transactional(readOnly = true)
    public SessionExtension findSession(Long id) {
        return sessionRepository.getSessionWithCapacity(id)
                .orElseThrow(() -> new SessionNotFoundException(id));
    }

    @Transactional(readOnly = true)
    public Session findBaseSession(Long id) {
        return sessionRepository.findById(id)
                .orElseThrow(() -> new SessionNotFoundException(id));
    }

    @Transactional(readOnly = true)
    public List<SessionExtension> findAllSessions() {
        return sessionRepository.getSessionsWithCapacity();
    }

    @Transactional
    public Session updateSession(Long id, Double price) {
        final Session currentSession = findBaseSession(id);
        currentSession.setPrice(price);
        validatorUtil.validate(currentSession);
        return sessionRepository.save(currentSession);
    }

    @Transactional
    public Session deleteSession(Long id) {
        final Session currentSession = findBaseSession(id);
        // все равно сеанс не удалился бы, который участвует в заказах
        // для отслеживания операции с ошибкой
        if (currentSession.getOrders().size() > 0)
            throw new IllegalArgumentException();
        sessionRepository.delete(currentSession);
        return currentSession;
    }

    @Transactional
    public void deleteAllSessions() {
        sessionRepository.deleteAll();
    }

    @Transactional
    public Integer getCapacity(Long sessionId) {
        return sessionRepository.getCapacity(sessionId).orElse(0);
    }
}
